@@ -43,6 +43,7 @@
 
 #include <librevenge/librevenge.h>
 
+#include "MWAWFontConverter.hxx"
 #include "MWAWListener.hxx"
 
 #include "MWAWCell.hxx"
@@ -198,6 +199,7 @@ bool MWAWCell::Format::convertDTFormat(std::string const &dtFormat, librevenge::
       break;
     case 'e':
       list.insert("number:style", "long");
+    // fall-through intended
     case 'd':
       list.insert("librevenge:value-type", "day");
       propVect.append(list);
@@ -211,6 +213,7 @@ bool MWAWCell::Format::convertDTFormat(std::string const &dtFormat, librevenge::
 
     case 'H':
       list.insert("number:style", "long");
+    // fall-through intended
     case 'I':
       list.insert("librevenge:value-type", "hours");
       propVect.append(list);
@@ -363,8 +366,8 @@ void MWAWCell::addTo(librevenge::RVNGPropertyList &propList, shared_ptr<MWAWFont
     propList.insert("fo:background-color", backgroundColor().str().c_str());
   if (isProtected())
     propList.insert("style:cell-protect","protected");
-  // alignement
-  switch (hAlignement()) {
+  // alignment
+  switch (hAlignment()) {
   case HALIGN_LEFT:
     propList.insert("fo:text-align", "first");
     propList.insert("style:text-align-source", "fix");
@@ -381,12 +384,12 @@ void MWAWCell::addTo(librevenge::RVNGPropertyList &propList, shared_ptr<MWAWFont
     break; // default
   case HALIGN_FULL:
   default:
-    MWAW_DEBUG_MSG(("MWAWCell::addTo: called with unknown halign=%d\n", hAlignement()));
+    MWAW_DEBUG_MSG(("MWAWCell::addTo: called with unknown halign=%d\n", hAlignment()));
   }
   // no padding
   propList.insert("fo:padding", 0, librevenge::RVNG_POINT);
-  // alignement
-  switch (vAlignement()) {
+  // alignment
+  switch (vAlignment()) {
   case VALIGN_TOP:
     propList.insert("style:vertical-align", "top");
     break;
@@ -399,7 +402,7 @@ void MWAWCell::addTo(librevenge::RVNGPropertyList &propList, shared_ptr<MWAWFont
   case VALIGN_DEFAULT:
     break; // default
   default:
-    MWAW_DEBUG_MSG(("MWAWCell::addTo: called with unknown valign=%d\n", vAlignement()));
+    MWAW_DEBUG_MSG(("MWAWCell::addTo: called with unknown valign=%d\n", vAlignment()));
   }
 }
 
@@ -413,7 +416,7 @@ std::string MWAWCell::getColumnName(int col)
   return f.str();
 }
 
-std::string MWAWCell::getCellName(Vec2i const &pos, Vec2b const &absolute)
+std::string MWAWCell::getCellName(MWAWVec2i const &pos, MWAWVec2b const &absolute)
 {
   std::stringstream f;
   f << "[.";
@@ -451,7 +454,7 @@ void MWAWCell::setBorders(int wh, MWAWBorder const &border)
 
 std::ostream &operator<<(std::ostream &o, MWAWCell const &cell)
 {
-  o << MWAWCell::getCellName(cell.m_position, Vec2b(false,false)) << ":";
+  o << MWAWCell::getCellName(cell.m_position, MWAWVec2b(false,false)) << ":";
   if (cell.numSpannedCells()[0] != 1 || cell.numSpannedCells()[1] != 1)
     o << "span=[" << cell.numSpannedCells()[0] << "," << cell.numSpannedCells()[1] << "],";
 
@@ -548,14 +551,51 @@ bool MWAWCell::sendContent(MWAWListenerPtr, MWAWTable &)
 ////////////////////////////////////////////////////////////
 bool MWAWCellContent::double2Date(double val, int &Y, int &M, int &D)
 {
-  // number of day since 1/1/1970
-  time_t date= time_t((val-24107-1462+0.4)*24.*3600);
-  struct tm dateTm;
-  if (!gmtime_r(&date,&dateTm)) return false;
+  /* first convert the date in long*/
+  long numDaysSinceOrigin=long(val+0.4);
+  // checkme: do we need to check before for isNan(val) ?
+  if (numDaysSinceOrigin<-10000*365 || numDaysSinceOrigin>10000*365) {
+    /* normally, we can expect documents to contain date between 1904
+       and 2004. So even if such a date can make sense, storing it as
+       a number of days is clearly abnormal */
+    MWAW_DEBUG_MSG(("MWAWCellContent::double2Date: using a double to represent the date %ld seems odd\n", numDaysSinceOrigin));
+    Y=1904;
+    M=D=1;
+    return false;
+  }
+  // find the century
+  int century=19;
+  while (numDaysSinceOrigin>=36500+24) {
+    long numDaysInCentury=36500+24+((century%4)?0:1);
+    if (numDaysSinceOrigin<numDaysInCentury) break;
+    numDaysSinceOrigin-=numDaysInCentury;
+    ++century;
+  }
+  while (numDaysSinceOrigin<0) {
+    --century;
+    numDaysSinceOrigin+=36500+24+((century%4)?0:1);
+  }
+  // now compute the year
+  Y=int(numDaysSinceOrigin/365);
+  long numDaysToEndY1=Y*365+(Y>0 ? (Y-1)/4+((century%4)?0:1) : 0);
+  if (numDaysToEndY1>numDaysSinceOrigin) {
+    --Y;
+    numDaysToEndY1=Y*365+(Y>0 ? (Y-1)/4+((century%4)?0:1) : 0);
+  }
+  // finish to compute the date
+  int numDaysFrom1Jan=int(numDaysSinceOrigin-numDaysToEndY1);
+  Y+=century*100;
+  bool isLeap=(Y%4)==0 && ((Y%400)==0 || (Y%100)!=0);
 
-  Y = dateTm.tm_year+1900;
-  M=dateTm.tm_mon+1;
-  D=dateTm.tm_mday;
+  for (M=0; M<12; ++M) {
+    static const int days[2][12] = {
+      { 0,31,59,90,120,151,181,212,243,273,304,334},
+      { 0,31,60,91,121,152,182,213,244,274,305,335}
+    };
+    if (M<11 && days[isLeap ? 1 : 0][M+1]<=numDaysFrom1Jan) continue;
+    D=(numDaysFrom1Jan-days[isLeap ? 1 : 0][M++])+1;
+    break;
+  }
   return true;
 }
 
@@ -585,7 +625,7 @@ bool MWAWCellContent::date2Double(int Y, int M, int D, double &val)
     { 0,31,60,91,121,152,182,213,244,274,305,335}
   };
   int32_t daysFrom1Jan=days[isLeap ? 1 : 0][M] + D;
-  val=double(daysFrom0-daysFrom1900+daysFrom1Jan)*24.*3600.;
+  val=double(daysFrom0-daysFrom1900+daysFrom1Jan);
   return true;
 }
 
@@ -720,7 +760,7 @@ std::ostream &operator<<(std::ostream &o, MWAWCellContent const &content)
 }
 
 // ---------- WKSContentListener::FormulaInstruction ------------------
-librevenge::RVNGPropertyList MWAWCellContent::FormulaInstruction::getPropertyList() const
+librevenge::RVNGPropertyList MWAWCellContent::FormulaInstruction::getPropertyList(MWAWFontConverter &fontConverter, int fontId) const
 {
   librevenge::RVNGPropertyList pList;
   switch (m_type) {
@@ -732,10 +772,26 @@ librevenge::RVNGPropertyList MWAWCellContent::FormulaInstruction::getPropertyLis
     pList.insert("librevenge:type","librevenge-function");
     pList.insert("librevenge:function",m_content.c_str());
     break;
-  case F_Text:
+  case F_Text: {
+    // we must use the font converter here to get the final string
     pList.insert("librevenge:type","librevenge-text");
-    pList.insert("librevenge:text",m_content.c_str());
+    librevenge::RVNGString finalStr("");
+    for (size_t i=0; i<m_content.size(); ++i) {
+      char c=m_content[i];
+      int unicode=fontConverter.unicode(fontId,(unsigned char)c);
+      if (unicode==-1) {
+        if (c < 0x20 && c!=9) {
+          MWAW_DEBUG_MSG(("MWAWCellContent::FormulaInstruction: Find odd char %x\n", (unsigned int)c));
+        }
+        else
+          finalStr.append(char(c));
+      }
+      else if (unicode != 0xfffd)
+        libmwaw::appendUnicode((uint32_t) unicode, finalStr);
+    }
+    pList.insert("librevenge:text",finalStr);
     break;
+  }
   case F_Double:
     pList.insert("librevenge:type","librevenge-number");
     pList.insert("librevenge:number",m_doubleValue, librevenge::RVNG_GENERIC);
