@@ -54,7 +54,7 @@
 #define DEBUG_BITMAP 0
 
 MWAWPictMac::ReadResult MWAWPictMac::checkOrGet
-(MWAWInputStreamPtr input, int size, Box2f &box, MWAWPictData **result)
+(MWAWInputStreamPtr input, int size, MWAWBox2f &box, MWAWPictData **result)
 {
   if (result) *result=0L;
 
@@ -103,7 +103,7 @@ MWAWPictMac::ReadResult MWAWPictMac::checkOrGet
     if (input->readULong(1) != 0xff) return MWAW_R_BAD;
   }
 
-  box.set(Vec2f(float(dim[1]),float(dim[0])), Vec2f(float(dim[3]),float(dim[2])));
+  box.set(MWAWVec2f(float(dim[1]),float(dim[0])), MWAWVec2f(float(dim[3]),float(dim[2])));
   if (!empty && (box.size().x() < 0 || box.size().y() < 0)) return MWAW_R_BAD;
   if (box.size().x() <= 0 || box.size().y() <= 0) empty = true;
 
@@ -181,7 +181,7 @@ public:
     }
     int val[4];
     for (int i = 0; i < 4; i++) val[i] = (int) input.readLong(2);
-    m_box.set(Vec2i(val[0], val[1]), Vec2i(val[2], val[3]));
+    m_box.set(MWAWVec2i(val[0], val[1]), MWAWVec2i(val[2], val[3]));
     sz -= 5;
     m_points.resize(0);
     if (sz == 0) return true;
@@ -216,7 +216,7 @@ public:
           MWAW_DEBUG_MSG(("Pict1:Region: found eroneous y value\n"));
           return false;
         }
-        m_points.push_back(Vec2i(x,y));
+        m_points.push_back(MWAWVec2i(x,y));
       }
       if (!endF) {
         MWAW_DEBUG_MSG(("Pict1:Region: does not find end of file...\n"));
@@ -233,9 +233,9 @@ public:
 
 protected:
   //! the bounding box
-  Box2i m_box;
+  MWAWBox2i m_box;
   //! the set of points which defines the mask
-  std::vector<Vec2i> m_points;
+  std::vector<MWAWVec2i> m_points;
 };
 
 //!  Internal and low level: a class used to read pack/unpack black-white bitmap
@@ -257,7 +257,7 @@ struct Bitmap {
     for (int c = 0; c < 3; c++) {
       int val[4];
       for (int d = 0; d < 4; d++) val[d] = (int) input.readLong(2);
-      Box2i box(Vec2i(val[1],val[0]), Vec2i(val[3],val[2]));
+      MWAWBox2i box(MWAWVec2i(val[1],val[0]), MWAWVec2i(val[3],val[2]));
       if (box.size().x() <= 0 || box.size().y() <= 0) {
         MWAW_DEBUG_MSG(("Pict1:Bitmap: find odd rectangle %d... \n", c));
         return false;
@@ -327,7 +327,7 @@ struct Bitmap {
   {
     if (m_rowBytes <= 0) return false;
     int nRows = int(m_bitmap.size())/m_rowBytes;
-    MWAWPictBitmapBW bitmap(Vec2i(m_rect.size().x(),nRows));
+    MWAWPictBitmapBW bitmap(MWAWVec2i(m_rect.size().x(),nRows));
     if (!bitmap.valid()) return false;
 
     for (int i = 0; i < nRows; i++)
@@ -419,7 +419,7 @@ struct Bitmap {
   }
   //! the num of bytes used to store a row
   int m_rowBytes;
-  Box2i m_rect /** the bitmap rectangle */, m_src/** the initial dimension */, /** another final dimension */ m_dst  ;
+  MWAWBox2i m_rect /** the bitmap rectangle */, m_src/** the initial dimension */, /** another final dimension */ m_dst  ;
   //! the region
   shared_ptr<Region> m_region;
   //! the bitmap
@@ -443,7 +443,7 @@ struct ColorTable {
     input.seek(4, librevenge::RVNG_SEEK_CUR); // ignore seed
     m_flags = (int) input.readULong(2);
     int n = (int) input.readLong(2)+1;
-    if (n < 0) return false;
+    if (n < 0 || !input.checkPosition(actPos+8+8*n)) return false;
     m_colors.resize(size_t(n));
     for (size_t i = 0; i < size_t(n); i++) {
       input.readULong(2); // indexId: ignored
@@ -497,7 +497,7 @@ struct Pixmap {
     // read the rectangle: bound
     int val[4];
     for (int d = 0; d < 4; d++) val[d] = (int) input.readLong(2);
-    m_rect = Box2i(Vec2i(val[1],val[0]), Vec2i(val[3],val[2]));
+    m_rect = MWAWBox2i(MWAWVec2i(val[1],val[0]), MWAWVec2i(val[3],val[2]));
     if (m_rect.size().x() <= 0 || m_rect.size().y() <= 0) {
       MWAW_DEBUG_MSG(("Pict1:Pixmap: find odd bound rectangle ... \n"));
       return false;
@@ -534,7 +534,7 @@ struct Pixmap {
       for (int c = 0; c < 2; c++) {
         int dim[4];
         for (int d = 0; d < 4; d++) dim[d] = (int) input.readLong(2);
-        Box2i box(Vec2i(dim[1],dim[0]), Vec2i(dim[3],dim[2]));
+        MWAWBox2i box(MWAWVec2i(dim[1],dim[0]), MWAWVec2i(dim[3],dim[2]));
         if (box.size().x() <= 0 || box.size().y() <= 0) {
           MWAW_DEBUG_MSG(("Pict1:Bitmap: find odd rectangle %d... \n", c));
           return false;
@@ -603,7 +603,7 @@ struct Pixmap {
     if (W <= 0) return false;
     if (m_colorTable.get() && m_indices.size()) {
       int nRows = int(m_indices.size())/W;
-      MWAWPictBitmapIndexed pixmap(Vec2i(W,nRows));
+      MWAWPictBitmapIndexed pixmap(MWAWVec2i(W,nRows));
       if (!pixmap.valid()) return false;
 
       pixmap.setColors(m_colorTable->m_colors);
@@ -625,7 +625,7 @@ struct Pixmap {
     }
     else if (m_colors.size()) {
       int nRows = int(m_colors.size())/W;
-      MWAWPictBitmapColor pixmap(Vec2i(W,nRows));
+      MWAWPictBitmapColor pixmap(MWAWVec2i(W,nRows));
       if (!pixmap.valid()) return false;
 
       size_t rPos = 0;
@@ -655,7 +655,10 @@ struct Pixmap {
   //! creates the pixmap from the packdata
   bool unpackedData(unsigned char const *pData, int sz, int byteSz, int nSize, std::vector<unsigned char> &res) const
   {
-    assert(byteSz >= 1 && byteSz <= 4);
+    if (byteSz<1||byteSz>4) {
+      MWAW_DEBUG_MSG(("libmwaw_applepict1::Pixmap::unpackedData: unknown byteSz\n"));
+      return false;
+    }
     int rPos = 0, wPos = 0, maxW = m_rowBytes+24;
     while (rPos < sz) {
       if (rPos+2 > sz) return false;
@@ -838,7 +841,7 @@ struct Pixmap {
 
   //! the num of bytes used to store a row
   int m_rowBytes;
-  Box2i m_rect /** the pixmap rectangle */;
+  MWAWBox2i m_rect /** the pixmap rectangle */;
   int m_version /** the pixmap version */;
   int m_packType /** the packing format */;
   long m_packSize /** size of data in the packed state */;
@@ -850,7 +853,7 @@ struct Pixmap {
   long m_planeBytes /** offset to the next plane */;
   shared_ptr<ColorTable> m_colorTable /** the color table */;
 
-  Box2i m_src/** the initial dimension */, /** another final dimension */ m_dst  ;
+  MWAWBox2i m_src/** the initial dimension */, /** another final dimension */ m_dst  ;
   //! the region
   shared_ptr<Region> m_region;
   //! the pixmap indices
@@ -1023,11 +1026,11 @@ struct Value {
   //! the text when type=WP_TEXT
   std::string m_text;
   //! the point when type=WP_POINT
-  Vec2i m_point;
+  MWAWVec2i m_point;
   //! the rectangle when type=WP_RECT
-  Box2i m_box;
+  MWAWBox2i m_box;
   //! the list of points which defined the polygon when type=WP_POLY
-  std::vector<Vec2i> m_listPoint;
+  std::vector<MWAWVec2i> m_listPoint;
   //! the region when type=WP_REGION
   shared_ptr<Region> m_region;
   //! the bitmap when type=WP_BITMAP
@@ -1044,7 +1047,11 @@ struct OpCode {
    *
    * \param id is the code of the opcode in the file
    * \param nm is the short name of the opcode
-   * \param type1 \param type2 \param type3 \param type4 \param type5 the type of the first, second, third arguments (if they exist)
+   * \param type1 type of the first component
+   * \param type2 type of the second component (if it exists)
+   * \param type3 type of the third component (if it exists)
+   * \param type4 type of the fourst component (if it exists)
+   * \param type5 type of the fifth component (if it exists)
    */
   OpCode(int id, char const *nm, DataType type1=WP_NONE, DataType type2=WP_NONE, DataType type3=WP_NONE, DataType type4=WP_NONE, DataType type5=WP_NONE)
     : m_id(id), m_name(nm), m_types()
@@ -1102,9 +1109,9 @@ struct OpCode {
   /** read a rectangles field
 
   \note can be used to read the first dimensions of a picture */
-  static bool readRect(MWAWInputStream &input, DataType type, Box2i &res)
+  static bool readRect(MWAWInputStream &input, DataType type, MWAWBox2i &res)
   {
-    Vec2i v[2];
+    MWAWVec2i v[2];
     DataType valType;
     switch (type) {
     case WP_RECT:
@@ -1303,7 +1310,7 @@ protected:
       // can not guess, so we read the pixmap...
       bool hasRgn = type ==WP_CRBITMAP;
       shared_ptr<Pixmap> pxmap(new Pixmap);
-      if (!pxmap->read(input, false, false, true, hasRgn)) return -1;
+      if (!pxmap->read(input, false, false, true, hasRgn)) return false;
       val.m_type = WP_CBITMAP;
       val.m_pixmap = pxmap;
       return true;
@@ -1469,7 +1476,7 @@ protected:
   }
 
   //! low level: reads a point argument
-  static bool readPoint(MWAWInputStream &input, DataType type, Vec2i &res)
+  static bool readPoint(MWAWInputStream &input, DataType type, MWAWVec2i &res)
   {
     int v[2];
     DataType valType;
@@ -1518,7 +1525,7 @@ protected:
   }
 
   //! low level: reads a polygon argument
-  static bool readPoly(MWAWInputStream &input, DataType type, Box2i &box, std::vector<Vec2i> &res)
+  static bool readPoly(MWAWInputStream &input, DataType type, MWAWBox2i &box, std::vector<MWAWVec2i> &res)
   {
     DataType boxType, valType;
     switch (type) {
@@ -1575,7 +1582,7 @@ protected:
     numPt /= 2;
     res.resize(size_t(numPt));
 
-    Vec2i pt;
+    MWAWVec2i pt;
     for (int p = 0; p < numPt; p++) {
       if (!readPoint(input, valType, pt)) return false;
       res[size_t(p)] = pt;
@@ -1626,14 +1633,15 @@ protected:
 
     long actualPos = input.tell();
     res = "";
+    if (!input.checkPosition(actualPos+sz)) {
+      MWAW_DEBUG_MSG(("Pict1:OpCode: readText: find EOF\n"));
+      return false;
+    }
+
     for (int i = 0; i < sz; i++) {
       char c = (char) input.readULong(1);
       res += c;
     }
-    if (actualPos+sz != input.tell()) {
-      MWAW_DEBUG_MSG(("Pict1:OpCode: readText: find EOF\n"));
-      return false;
-    }
     return true;
   }
 
@@ -1753,7 +1761,7 @@ void PictParser::parse(MWAWInputStreamPtr input, libmwaw::DebugFile &dFile)
   dFile.addNote(s.str().c_str());
   actPos = 2;
 
-  Box2i box;
+  MWAWBox2i box;
   bool ok = OpCode::readRect(*input, WP_RECT, box);
   if (ok) {
     s.str("");
@@ -1768,7 +1776,7 @@ void PictParser::parse(MWAWInputStreamPtr input, libmwaw::DebugFile &dFile)
     int code = (int) input->readULong(1);
     std::map<int,OpCode const *>::iterator it = m_mapIdOp.find(code);
     if (it == m_mapIdOp.end() || it->second == 0L) {
-      MWAW_DEBUG_MSG(("Pict1:OpCode:parsePict can not find opCode 0x%x\n", code));
+      MWAW_DEBUG_MSG(("Pict1:OpCode:parsePict can not find opCode 0x%x\n", (unsigned int) code));
       input->seek(actPos, librevenge::RVNG_SEEK_SET);
       ok = false;
       break;
@@ -1777,7 +1785,7 @@ void PictParser::parse(MWAWInputStreamPtr input, libmwaw::DebugFile &dFile)
     OpCode const &opCode = *(it->second);
     std::vector<Value> readData;
     if (!opCode.readData(*input, readData)) {
-      MWAW_DEBUG_MSG(("Pict1:OpCode:parsePict error for opCode 0x%x\n", code));
+      MWAW_DEBUG_MSG(("Pict1:OpCode:parsePict error for opCode 0x%x\n", (unsigned int) code));
       input->seek(actPos, librevenge::RVNG_SEEK_SET);
       ok = false;
       break;
@@ -1856,7 +1864,7 @@ bool PictParser::convertToPict2(librevenge::RVNGBinaryData const &orig, libreven
     int code = (int) input->readULong(1);
     std::map<int,OpCode const *>::iterator it = m_mapIdOp.find(code);
     if (it == m_mapIdOp.end() || it->second == 0L) {
-      MWAW_DEBUG_MSG(("Pict1:convertToPict2 can not find opCode 0x%x\n", code));
+      MWAW_DEBUG_MSG(("Pict1:convertToPict2 can not find opCode 0x%x\n", (unsigned int) code));
       delete [] res;
       return false;
     }
@@ -1864,7 +1872,7 @@ bool PictParser::convertToPict2(librevenge::RVNGBinaryData const &orig, libreven
     OpCode const &opCode = *(it->second);
     sz = 0;
     if (!opCode.computeSize(*input, sz)) {
-      MWAW_DEBUG_MSG(("Pict1:convertToPict2 can not compute size for opCode 0x%x\n", code));
+      MWAW_DEBUG_MSG(("Pict1:convertToPict2 can not compute size for opCode 0x%x\n", (unsigned int) code));
       delete [] res;
       return false;
     }
@@ -1915,47 +1923,6 @@ namespace libmwaw_applepict2
 {
 using namespace libmwaw_applepict1;
 
-//! Internal and low level: a class used to read and store all possible value
-struct Value : public libmwaw_applepict1::Value {
-};
-
-//! Internal and low level: a class to define each opcode and their arguments and read their data
-struct OpCode : public libmwaw_applepict1::OpCode {
-  typedef libmwaw_applepict1::OpCode parent;
-  /** constructor
-   *
-   * \param id is the code of the opcode in the file
-   * \param nm is the short name of the opcode
-   * \param type1 \param type2 \param type3 \param type4 \param type5 the type of the first, second, third arguments (if they exist)
-   */
-  OpCode(int id, char const *nm, DataType type1=WP_NONE, DataType type2=WP_NONE, DataType type3=WP_NONE, DataType type4=WP_NONE, DataType type5=WP_NONE) :
-    parent(id, nm, type1, type2, type3, type4, type5) {}
-
-  /** tries to read the data in the file
-   *
-   * If the read is succefull, fills listValue with the read argument */
-  bool readData(MWAWInputStream &input, std::vector<Value> &listValue) const
-  {
-    size_t numTypes = m_types.size();
-    listValue.resize(numTypes);
-    Value newVal;
-    long debPos = input.tell();
-    for (size_t i = 0; i < numTypes; i++) {
-      long actualPos = input.tell();
-      if (readValue(input, m_types[i], newVal)) {
-        listValue[i] = newVal;
-        continue;
-      }
-      input.seek(actualPos, librevenge::RVNG_SEEK_SET);
-      return false;
-    }
-    long actualPos = input.tell();
-    // we must check alignement
-    if ((actualPos - debPos)%2 == 1) input.seek(1, librevenge::RVNG_SEEK_CUR);
-    return true;
-  }
-};
-
 /** internal and low level: list of new opcodes */
 static OpCode const s_listCodes[] = {
   OpCode(0x12,"BackCPat",WP_CPATTERN), OpCode(0x13,"PenCPat",WP_CPATTERN), OpCode(0x14,"FillCPat",WP_CPATTERN),
@@ -1978,9 +1945,9 @@ public:
   //! the constructor
   PictParser() : m_mapIdOp()
   {
-    size_t numCodes = sizeof(libmwaw_applepict1::s_listCodes)/sizeof(libmwaw_applepict1::OpCode);
+    size_t numCodes = sizeof(libmwaw_applepict1::s_listCodes)/sizeof(OpCode);
     for (size_t i = 0; i < numCodes; i++)
-      m_mapIdOp[libmwaw_applepict1::s_listCodes[i].m_id] = static_cast<OpCode const *>(&(libmwaw_applepict1::s_listCodes[i]));
+      m_mapIdOp[libmwaw_applepict1::s_listCodes[i].m_id] = &(libmwaw_applepict1::s_listCodes[i]);
     numCodes = sizeof(s_listCodes)/sizeof(OpCode);
     for (size_t i = 0; i < numCodes; i++)
       m_mapIdOp[s_listCodes[i].m_id] = &(s_listCodes[i]);
@@ -2017,7 +1984,7 @@ void PictParser::parse(MWAWInputStreamPtr input, libmwaw::DebugFile &dFile)
   dFile.addNote(s.str().c_str());
   actPos = 2;
 
-  Box2i box;
+  MWAWBox2i box;
   bool ok = OpCode::readRect(*input, WP_RECT, box);
   if (ok) {
     s.str("");
@@ -2094,7 +2061,7 @@ void PictParser::parse(MWAWInputStreamPtr input, libmwaw::DebugFile &dFile)
     int code = (int)input->readULong(2);
     std::map<int,OpCode const *>::iterator it = m_mapIdOp.find(code);
     if (it == m_mapIdOp.end() || it->second == 0L) {
-      MWAW_DEBUG_MSG(("Pict2:OpCode:parsePict can not find opCode 0x%x\n", code));
+      MWAW_DEBUG_MSG(("Pict2:OpCode:parsePict can not find opCode 0x%x\n", (unsigned int) code));
       input->seek(actPos, librevenge::RVNG_SEEK_SET);
       ok = false;
       break;
@@ -2103,11 +2070,15 @@ void PictParser::parse(MWAWInputStreamPtr input, libmwaw::DebugFile &dFile)
     OpCode const &opCode = *(it->second);
     std::vector<Value> readData;
     if (!opCode.readData(*input, readData)) {
-      MWAW_DEBUG_MSG(("Pict2:OpCode:parsePict error for opCode 0x%x\n", code));
+      MWAW_DEBUG_MSG(("Pict2:OpCode:parsePict error for opCode 0x%x\n", (unsigned int) code));
       input->seek(actPos, librevenge::RVNG_SEEK_SET);
       ok = false;
       break;
     }
+    // we must check alignment
+    if ((input->tell() - actPos)%2 == 1)
+      input->seek(1, librevenge::RVNG_SEEK_CUR);
+
     s.str("");
     s << opCode.m_name << ":";
     for (size_t i = 0; i < readData.size(); i++) {
